import { useState, useEffect } from "react";
import type { RSVPItem } from "../lib/types";
import { fetchWithAuth, getAuthToken } from "../utils/auth-client";

const RSVPManager = () => {
  const [rsvpList, setRSVPList] = useState<RSVPItem[]>([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  const fetchRSVPList = async () => {
    // Don't try to fetch if we don't have a token yet
    if (!getAuthToken()) {
      setError("No authentication token found");
      setLoading(false);
      return;
    }

    setLoading(true);
    setError(null);
    try {
      const response = await fetchWithAuth("/api/rsvp");
      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }
      const data = await response.json();
      setRSVPList(data);
      setError(null);
    } catch (e: any) {
      setError(e.message);
      console.error("Failed to fetch RSVP list:", e);
    } finally {
      setLoading(false);
    }
  };

  useEffect(() => {
    fetchRSVPList();

    // Add listener for auth success to retry fetching
    const handleAuthSuccess = () => {
      fetchRSVPList();
    };
    document.addEventListener("auth-success", handleAuthSuccess);
    return () => document.removeEventListener("auth-success", handleAuthSuccess);
  }, []);

  if (loading) {
    return <div className="text-center">Loading RSVP list...</div>;
  }

  if (error) {
    if (error === "No authentication token found") {
      return <div className="text-center">Initializing...</div>;
    }
    return <div className="text-red-500 text-center">Error: {error}</div>;
  }

  return (
    <div className="container mx-auto p-4">
      <div className="overflow-x-auto">
        <table className="table w-full">
          <thead>
            <tr>
              <th>Name</th>
              <th>Dietary Restrictions</th>
              <th>Notes</th>
              <th>Timestamp</th>
            </tr>
          </thead>
          <tbody>
            {rsvpList.map((rsvp, index) => (
              <tr key={index}>
                <td>{rsvp.name}</td>
                <td>{rsvp.dietaryRestrictions || "None"}</td>
                <td>{(rsvp.notes && rsvp.notes !== "undefined") ? rsvp.notes.trim() : "None"}</td>
                <td>{new Date(rsvp.timestamp).toLocaleString()}</td>
              </tr>
            ))}
          </tbody>
        </table>
      </div>
      
      <div className="mt-4 text-center">
        <p>Total RSVPs: {rsvpList.length}</p>
      </div>
    </div>
  );
};

export default RSVPManager; 