import { useState, useEffect } from "react";
import type { RegistryItem } from "../lib/types";
import { fetchWithAuth } from "../utils/auth-client";

const RegistryList = () => {
	const [registryItems, setRegistryItems] = useState<RegistryItem[]>([]);
	const [loading, setLoading] = useState(true);
	const [error, setError] = useState<string | null>(null);
	const [claimantName, setClaimantName] = useState("");
	const [claimedItems, setClaimedItems] = useState<Set<string>>(new Set());

	useEffect(() => {
		fetchRegistryItems();
	}, []);

	const fetchRegistryItems = async () => {
		setLoading(true);
		setError(null);
		try {
			const response = await fetchWithAuth("/api/registry");
			if (!response.ok) {
				throw new Error(`HTTP error! status: ${response.status}`);
			}
			const data = await response.json();
			setRegistryItems(data);
			if (data.length === 0) {
				document.dispatchEvent(new Event("registry-empty"));
			}
		} catch (e: any) {
			setError(e.message);
			console.error("Failed to fetch registry items:", e);
		} finally {
			setLoading(false);
		}
	};

	const handleItemClick = (itemId: string) => {
		const newClaimedItems = new Set(claimedItems);
		if (newClaimedItems.has(itemId)) {
			newClaimedItems.delete(itemId);
		} else {
			newClaimedItems.add(itemId);
		}
		setClaimedItems(newClaimedItems);
	};

	const handleSubmit = async () => {
		if (!claimantName.trim()) {
			setError("Please enter your name before claiming items");
			return;
		}

		const updates = registryItems.filter((item) => claimedItems.has(item.id));

		const updatedRegistryItems = registryItems.map((item) => {
			if (claimedItems.has(item.id)) {
				return { ...item, taken: true, claimedBy: claimantName };
			}
			return item;
		});
		setRegistryItems(updatedRegistryItems);
		setClaimedItems(new Set());
		setClaimantName("");

		try {
			for (const item of updates) {
				const response = await fetchWithAuth(`/api/registry/${item.id}`, {
					method: "PUT",
					headers: {
						"Content-Type": "application/json",
					},
					body: JSON.stringify({ taken: true, claimedBy: claimantName }),
				});

				if (!response.ok) {
					throw new Error(
						`Failed to update item ${item.id}: ${response.statusText}`
					);
				}
			}
			alert("Thank you for claiming items!");
		} catch (error: any) {
			console.error("Error updating items:", error);
			setError("Failed to update items. Please try again.");
			setRegistryItems(registryItems);
		}
	};

	if (loading) {
		return <div className="text-center">Loading registry items...</div>;
	}

	if (error) {
		return <div className="text-red-500 text-center">Error: {error}</div>;
	}

	const availableItems = registryItems.filter((item) => !item.taken);
	const claimedItemsArray = registryItems.filter((item) => item.taken);

	return (
		<div className="container mx-auto p-4">
			{availableItems.length > 0 && (
				<>
					<div className="mb-4">
						<input
							type="text"
							value={claimantName}
							onChange={(e) => setClaimantName(e.target.value)}
							placeholder="Enter your name"
							className="input input-bordered w-full max-w-xs"
						/>
					</div>

					<div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-4 mb-8">
						{availableItems.map((item) => (
							<div
								key={item.id}
								className={`card bg-base-100 shadow-xl cursor-pointer ${
									claimedItems.has(item.id)
										? "border-4 border-primary"
										: ""
								}`}
								onClick={() => handleItemClick(item.id)}
							>
								<div className="card-body">
									<h2 className="card-title">{item.name}</h2>
									{item.link && (
										<a
											href={item.link}
											target="_blank"
											rel="noopener noreferrer"
											className="link link-primary"
											onClick={(e) => e.stopPropagation()}
										>
											View Item
										</a>
									)}
								</div>
							</div>
						))}
					</div>

					{claimedItems.size > 0 && (
						<div className="text-center mb-8">
							<button
								onClick={handleSubmit}
								className="btn btn-primary"
								disabled={!claimantName.trim()}
							>
								Claim Selected Items
							</button>
						</div>
					)}
				</>
			)}

			{claimedItemsArray.length > 0 && (
				<div className="mt-8">
					<h2 className="text-2xl font-bold mb-4">Already Claimed</h2>
					<div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-4">
						{claimedItemsArray.map((item) => (
							<div
								key={item.id}
								className="card bg-base-100 shadow-xl opacity-50"
							>
								<div className="card-body">
									<h2 className="card-title">{item.name}</h2>
									<p className="text-sm">
										Claimed by: {item.claimedBy}
									</p>
								</div>
							</div>
						))}
					</div>
				</div>
			)}
		</div>
	);
};

export default RegistryList;
